@@ -5,7 +5,7 @@
 $::PROTOCOL = '(?:s?https?|ftp)';
 $::RXURL    = '(?:s?https?|ftp)://[-\\w.!~*\'();/?:@&=+$,%#]+' ;
 $::charset  = 'EUC-JP';
-$::version  = '1.1.8_2';
+$::version  = '1.1.9_0';
 %::form     = %::forms = ();
 $::me       = $::postme = $ENV{SCRIPT_NAME};
 $::print    = ' 'x 10000; $::print = '';
@@ -163,7 +163,7 @@ sub init_globals{
 
     %::preferences = (
         ' General Options' => [
-            { desc=>'script-revision '.$::version.' $Date: 2007/06/03 17:48:40 $' ,
+            { desc=>'script-revision '.$::version.' $Date: 2007/08/16 06:38:25 $' ,
               type=>'rem' },
             { desc=>'The sitename', name=>'sitename', size=>40 },
             { desc=>'Enable link to file://...', name=>'locallink' ,
@@ -488,8 +488,9 @@ sub print_header{
     &puts( @::body_header );
     if( $arg{userheader} ){
         &putenc('<div class="%s">' , $arg{divclass}||'main' );
-        &print_page( title=>'Header' , class=>'header' );
+        my $r=&print_page( title=>'Header' , class=>'header' );
         &puts( &plugin({},'menubar') ) unless $::flag{menubar_printed} ;
+        &putenc('<h1>%s</h1>',$::config{sitename}) if !$r && $::config{sitename};
         $::flag{userheader} = 1;
     }else{
         &putenc('<div class="%s">' , $arg{divclass}||'max' );
@@ -995,7 +996,7 @@ sub cache_update{
         while( my $fn=readdir(DIR) ){
             push( @::dir_cache , $fn );
             if( $fn =~ /^((?:[0-9a-f][0-9a-f])+)(__(?:[0-9a-f][0-9a-f])+)?$/ ){
-                push( @{$::dir_cache{$1}} , $2 );
+                push( @{$::dir_cache{$1}} , $2||'' );
             }
         }
         closedir(DIR);
@@ -1265,6 +1266,7 @@ sub plugin_comment{
         close(FP);
     }
     unless( exists $opt{f} ){
+        my $readonly=(($::form{a}||0)eq'Preview'?' readonly':'');
         $buf .= <<HTML
 <div class="form">
 <form action="$::postme" method="post" class="comment">
@@ -1272,10 +1274,10 @@ sub plugin_comment{
 <input type="hidden" name="a" value="comment">
 <input type="hidden" name="comid" value="$ecomid">
 <div class="field name">
-<input type="text" name="who" size="10" class="field">
+<input type="text" name="who" size="10" class="field"$readonly>
 </div><!-- div.field name -->
 <div class="textarea">
-<textarea name="comment" cols="60" rows="1" class="field"></textarea>
+<textarea name="comment" cols="60" rows="1" class="field"$readonly></textarea>
 </div><!-- div.textarea -->
 <div class="button">
 <input type="submit" name="Comment" value="Comment">
@@ -1303,12 +1305,17 @@ sub plugin{
     my $session=shift;
     my ($name,$param)=(split(/\s+/,shift,2),'');
     $session->{argv} = $param;
+    
+    $param =~ s/\x02.*?\x02/"\x05".unpack('h*',$&)."\x05"/eg;
+    my @param=split(/\s+/,$param);
+    grep{
+        s|\x05([^\x05]*)\x05|pack('h*',$1)|ge;
+        s|\x02+|"\x02"x(length($&)>>1)|ge;
+        0;
+    } @param;
 
-    if( exists $::inline_plugin{$name} ){
-        $::inline_plugin{$name}->($session,split(/\s+/,$param)) || '';
-    }else{
-        'Plugin not found.';
-    }
+    ($::inline_plugin{$name} || sub{'Plugin not found.'} )
+        ->($session,@param) || '';
 }
 
 sub cr2br{
@@ -1318,14 +1325,15 @@ sub cr2br{
     $s;
 }
 
-sub preprocess_innerlink1{ ### >>{ ... } ###
+sub preprocess_innerlink1{ ### >>{ ... }{ ... } ###
     my ($text,$session)=@_;
-    $$text =~ s|&gt;&gt;\{([^\}]+)\}|&inner_link($session,$1,$1)|ge;
+    $$text =~ s|&gt;&gt;\{([^\}]+)\}(?:\{([^\}]*)\})?|
+        &inner_link($session,defined($2)?$2:$1,$1)|ge;
 }
 
 sub preprocess_innerlink2{ ### [[ ... | ... ] ###
     my ($text,$session)=@_;
-    ${$_[0]} =~ s!\[\[(?:([^\|\]]+)\|)?(.+?)\]\]!
+    $$text =~ s!\[\[(?:([^\|\]]+)\|)?(.+?)\]\]!
         &inner_link($session,defined($1)?$1:$2,$2)!ge;
 }
 
@@ -1377,7 +1385,14 @@ sub preprocess_decorations{
 }
 
 sub preprocess_plugin{
-    ${$_[0]} =~ s/\(\((.+?)\)\)/&plugin($_[1],$1)/ges;
+    my ($text,$sesion) = @_;
+    $$text =~ s/&quot;/\x02/g;
+    $$text =~ s/\(\(/\x03/g;
+    $$text =~ s/\)\)/\x04/g;
+    $$text =~ s/\x03([^\x02-\x04]*?(?:\x02[^\x02]*\x02[^\x02-\x04]*?)*?)\x04/&plugin($sesion,$1)/ges;
+    $$text =~ s/\x04/\)\)/g;
+    $$text =~ s/\x03/\(\(/g;
+    $$text =~ s/\x02/&quot;/g;
 }
 
 sub preprocess_rawurl{
